import os
import slth
import json
import types
from decimal import Decimal
from datetime import date, datetime
from django.template.loader import render_to_string
from django.db.models import Model, QuerySet, Manager
from django.utils.text import slugify
from .exceptions import JsonResponseException
from .utils import absolute_url, build_url
from .components import Image, FileLink, FileViewer
from django.db.models.fields.files import ImageFieldFile, FieldFile


def getattrr(obj, args):
    return getattr_rec(obj, args.split('__'))


def getattr_rec(obj, args):
    if obj is None:
        return None
    if len(args) > 1:
        attr = args.pop(0)
        return getattr_rec(getattrr(obj, attr), args)
    else:
        return getattr(obj, args[0])


def to_snake_case(name):
    return slugify(name).replace('-', '_')


def serialize(obj, primitive=False, request=None):
    if obj is None:
        return None
    elif isinstance(obj, dict):
        if request:
            if isinstance(obj, Image):
                if isinstance(obj['src'], ImageFieldFile):
                    obj['src'] = build_url(request, obj['src'].url) if obj['src'] else None
            elif isinstance(obj, FileLink) or isinstance(obj, FileViewer):
                if isinstance(obj['url'], FieldFile):
                    obj['url'] = build_url(request, obj['url'].url) if obj['url'] else None
        return obj
    elif isinstance(obj, datetime):
        return obj.strftime('%d/%m/%Y %H:%M:%S')
    elif isinstance(obj, date):
        return obj.strftime('%d/%m/%Y')
    elif isinstance(obj, bool):
        return obj
    if isinstance(obj, Decimal) or isinstance(obj, float):
        return str(obj).replace('.', ',')
    elif isinstance(obj, list):
        return [serialize(obj) for obj in obj]
    elif isinstance(obj, Model):
        return str(obj) if primitive else dict(pk=obj.pk, str=str(obj))
    elif isinstance(obj, QuerySet) or isinstance(obj, Manager) or type(obj).__name__ == 'ManyRelatedManager':
        return [str(obj) for obj in obj.filter()] if primitive else obj.serialize()
    elif isinstance(obj, ImageFieldFile) or isinstance(obj, FieldFile):
        return str(obj)
    elif hasattr(obj, 'serialize'):
        return obj.serialize()
    return str(obj)

def getfield(obj, name_or_names, request=None):
    if isinstance(name_or_names, str):
        if ':' in name_or_names:
            name_or_names, action_name = name_or_names.split(':')
        else:
            name_or_names, action_name = name_or_names, None
        attr = getattrr(obj, name_or_names)
        if type(attr) == types.MethodType:
            value = attr()
            label = getattr(attr, 'verbose_name', name_or_names.replace('get_', ''))
        elif name_or_names.startswith('get_') and name_or_names.endswith('_display'):
            value = attr()
            label = getattr(type(obj), name_or_names[4:-8]).field.verbose_name
        else:
            value = attr
            label = type(obj).get_field(name_or_names).verbose_name
        label = label.title().replace('_', ' ') if label and label.islower() else label
        field = dict(type='field', name=name_or_names, label=label, value=serialize(value, primitive=True, request=request))
        if action_name:
            cls = slth.ENDPOINTS[action_name]
            endpoint = cls.instantiate(request, obj)
            if endpoint.check_permission():
                field.update(url=endpoint.get_api_url(obj.id))
        return field
    else:
        fields = []
        for name in name_or_names:
            fields.append(getfield(obj, name, request))
        return fields


class Serializer:
    def __init__(self, obj=None, request=None, serializer=None, type='object', title=None, show_title=True):
        self.path = serializer.path.copy() if serializer else []
        self.obj = obj
        self.lazy = False
        self.ignore_only = False
        self.request = request
        self.metadata = dict(actions=[], content=[], allow=[])
        self.serializer:Serializer = serializer
        self.type = type
        if title:
            self.title = title
            self.path.append(to_snake_case(title))
        elif isinstance(obj, Model):
            self.title = str(obj)
        else:
            self.title = None
        self.show_title = show_title

    def actions(self, *actions) -> 'Serializer':
        self.metadata['actions'].extend(actions)
        self.metadata['allow'].extend(actions)
        return self
    
    def field(self, name, reload=True, condition=None, roles=()) -> 'Serializer':
        self.metadata['content'].append(('field', name, dict(name=name, reload=reload, condition=condition, roles=roles)))
        return self
        
    def fields(self, *names, condition=None, roles=()) -> 'Serializer':
        self.metadata['content'].append(('fields', None, dict(names=names, condition=condition, roles=roles)))
        return self
    
    def fieldset(self, title, fields=(), actions=(), attr=None, reload=True, condition=None, roles=()) -> 'Serializer':
        self.metadata['allow'].extend(actions)
        item = dict(title=title, names=fields, attr=attr, actions=actions, reload=reload, condition=condition, roles=roles)
        self.metadata['content'].append(('fieldset', to_snake_case(title), item))
        return self
        
    def queryset(self, title, name, condition=None, roles=()) -> 'Serializer':
        self.metadata['content'].append(('queryset', name, dict(title=title, condition=condition, roles=roles)))
        return self
    
    def endpoint(self, title, cls, wrap=True, condition=None, roles=()) -> 'Serializer':
        if isinstance(cls, str):
            cls = slth.ENDPOINTS[cls]
        self.metadata['content'].append(('endpoint', to_snake_case(title), dict(title=title, cls=cls, wrap=wrap, condition=condition, roles=roles)))
        return self
    
    def append(self, title, component, condition=None, roles=()) -> 'Serializer':
        self.metadata['content'].append(('component', to_snake_case(title), dict(title=title, component=component, condition=condition, roles=roles)))
    
    def section(self, title) -> 'Serializer':
        return Serializer(obj=self.obj, request=self.request, serializer=self, type='section', title=title)
    
    def group(self, title=None) -> 'Serializer':
        show_title = True
        if title is None:
            title = 'group'
            show_title = False
        return Serializer(obj=self.obj, request=self.request, serializer=self, type='group', title=title, show_title=show_title)

    def parent(self) -> 'Serializer':
        self.serializer.metadata['content'].append(('serializer', to_snake_case(self.title), dict(serializer=self, condition=None, roles=())))
        return self.serializer
    
    def contextualize(self, request) -> 'Serializer':
        self.request = request
        return self
    
    def settitle(self, title) -> 'Serializer':
        self.title = title
        return self
    
    def serialize(self, debug=False, forward_exception=False):
        try:
            return self.to_dict(debug=debug)
        except JsonResponseException as e:
            if self.serializer or forward_exception:
                raise e
            if debug:
                print(json.dumps(e.data, indent=2, ensure_ascii=False))
            return e.data
        
    def __str__(self):
        return f'{self.title} / {self.type}'

    def check_condition(self, name):
        if name:
            deny = False
            if name.startswith('not '):
                deny, name = name.split()
            attr = getattr(self.obj, name)
            if callable(attr):
                attr = attr()
            return not attr if deny else attr
        return True
    
    def check_role(self, names):
        from slth.models import Role
        return self.request.user.is_superuser or not names or Role.objects.filter(username=self.request.user.username, name__in=names).exists()

    def to_dict(self, debug=False):
        base_url = absolute_url(self.request)
        if self.request is None and self.serializer:
            self.request = self.serializer.request
        if self.ignore_only:
            only = []
        else:
            only = self.request and self.request.GET.get('only', None)
            if only:
                only = only.split('__')

        if self.request and 'action' in self.request.GET:
            cls = slth.ENDPOINTS[self.request.GET.get('action')]
            if cls and cls.get_api_name() in self.metadata['allow']:
                endpoint = cls.instantiate(self.request, self.obj)
                if endpoint.check_permission():
                    raise JsonResponseException(endpoint.serialize())
                
        actions = []
        if (not only or 'actions' in only) and not self.lazy:
            leaf = only and only[-1] == 'actions'
            for qualified_name in self.metadata['actions']:
                cls = slth.ENDPOINTS[qualified_name]
                if cls.instantiate(self.request, self.obj).check_permission():
                    actions.append(cls.get_api_metadata(self.request, base_url, self.obj.pk))
            if leaf:
                raise JsonResponseException(actions)

        if not self.metadata['content'] and self.obj:
            self.fields(*[field.name for field in type(self.obj)._meta.fields])
            for m2m in type(self.obj)._meta.many_to_many:
                self.queryset(m2m.verbose_name, m2m.name)
        
        items = []
        if not self.lazy:
            for i, (datatype, key, item) in enumerate(self.metadata['content']):
                if not self.check_condition(item['condition']):
                    continue
                if not self.check_role(item['roles']):
                    continue
                leaf = only and only[-1] == key
                lazy = i and self.type == 'group' and not leaf
                data = None
                if datatype == 'field':
                    path = self.path + [item['name']]
                    data = getfield(self.obj, item['name'], self.request)
                    if item['reload']:
                        data['url'] = absolute_url(self.request, 'only={}'.format('__'.join(path)))
                    if leaf: raise JsonResponseException(data)
                elif datatype == 'fields':
                    data = []
                    if not lazy:
                        for name in item['names']:
                            if key is None or not only or name in only:
                                data.append(getfield(self.obj, name, self.request))
                        if only and only[-1] == name: raise JsonResponseException(data)
                elif datatype == 'fieldset':
                    title = item['title']
                    names = item['names']
                    attr = item['attr']
                    reload = item['reload']
                    if not only or key in only:
                        fieldset_actions=[]
                        fieldset_fields=[]
                        title = title.title() if title and title.islower() else title
                        url = absolute_url(self.request, 'only={}'.format('__'.join(self.path + [key])))
                        if lazy:
                            data = dict(type='fieldset', title=title, key=key, url=url)
                        else:
                            obj = getattr(self.obj, attr) if attr else self.obj
                            if obj:
                                for name in names:
                                    fieldset_fields.append(getfield(obj, name, self.request))
                            for qualified_name in item['actions']:
                                cls = slth.ENDPOINTS[qualified_name]
                                if cls.instantiate(self.request, self.obj).check_permission():
                                    fieldset_actions.append(cls.get_api_metadata(self.request, base_url, self.obj.pk))
                            data = dict(type='fieldset', title=title, key=key, url=url if reload else None, actions=fieldset_actions, data=fieldset_fields)
                        if leaf: raise JsonResponseException(data)
                elif datatype == 'queryset':
                    title = item['title']
                    if not only or key in only:
                        attr = getattr(self.obj, key)
                        if type(attr) == types.MethodType:
                            value = attr().filter()
                        else:
                            value = attr.filter()
                    
                        path = self.path + [key]
                        if lazy:
                            data = dict(type='queryset', title=title, key=key)
                        else:
                            data = value.settitle(title).attrname('__'.join(path)).contextualize(self.request).serialize(debug=False)
                        
                        data['url'] = absolute_url(self.request, 'only={}'.format('__'.join(path)))
                        if leaf: raise JsonResponseException(data)
                elif datatype == 'endpoint':
                    data = {}
                    title = item['title']
                    cls = item['cls']
                    wrap = item['wrap']
                    if not only or key in only:
                        returned = {}
                        endpoint = cls.instantiate(self.request, self.obj)
                        if endpoint.check_permission():
                            if not lazy:
                                returned = endpoint.process()
                            path = self.path + [key]
                            if wrap:
                                data = dict(type='fieldset', key=key, title=title, url=None, data=serialize(returned))
                            else:
                                data = serialize(returned)
                                data['title'] = title
                            if isinstance(data, dict):
                                data['url'] = absolute_url(self.request, 'only={}'.format('__'.join(path)))
                            if leaf: raise JsonResponseException(data)
                elif datatype == 'component':
                    title = item['title']
                    component = item['component']
                    data = {}
                    if not only or key in only:
                        data = serialize(component)
                    path = self.path + [key]
                    data['url'] = absolute_url(self.request, 'only={}'.format('__'.join(path)))
                    if leaf: raise JsonResponseException(data)
                elif datatype == 'serializer':
                    serializer = item['serializer']
                    serializer.lazy = lazy
                    if not only or key in only:
                        serializer.lazy = False
                        serializer.ignore_only = self.ignore_only or leaf
                        data = serializer.to_dict()
                        if leaf: raise JsonResponseException(data)
                if data:
                    items.extend(data) if datatype == 'fields' else items.append(data)

        output = dict(type=self.type, title=self.title if self.show_title else None)
        if self.serializer:
            datatype = to_snake_case(self.title)
            output.update(key=datatype)
        if self.path:
            url = absolute_url(self.request, 'only={}'.format('__'.join(self.path)))
        else:
            url = absolute_url(self.request)
        output.update(url=url)
        if not self.lazy:
            output.update(actions=actions, data=items)
        if debug:
            print(json.dumps(output, indent=2, ensure_ascii=False))
        return output



import json
from typing import Any
import datetime
from django.forms import *
from django.utils.translation import gettext_lazy as _
from django.forms.models import ModelChoiceIterator, ModelMultipleChoiceField
from django.db.models import Model, QuerySet, Manager
from .models import Token, Profile
from django.db import transaction
from django.db.models import Manager
from .exceptions import JsonResponseException
from .utils import absolute_url, build_url
from .serializer import Serializer
from slth import ENDPOINTS
from .components import Scheduler


DjangoModelForm = ModelForm
DjangoForm = Form
DjangoModelChoiceField = ModelChoiceField
DjangoMultipleChoiceField = MultipleChoiceField
DjangoModelMultipleChoiceField = ModelMultipleChoiceField
DjangoFileField = FileField
DjangoImageField = ImageField

MASKS = dict(
    cpf_cnpj="999.999.999-99|99.999.999/9999-99",
    cpf="999.999.999-99",
    cnpj="99.999.999/9999-99",
    telefone="(99) 99999-9999",
)


class FormController:

    def __init__(self, form):
        super().__init__()
        self.form = form
        self.controls = dict(hide=[], show=[], reload=[], set={})

    def hide(self, *names):
        self.controls["hide"].extend(names)

    def show(self, *names):
        self.controls["show"].extend(names)

    def reload(self, *names):
        self.controls["reload"].extend(names)

    def set(self, **kwargs):
        for k, v in kwargs.items():
            if isinstance(v, Model):
                v = dict(id=v.id, value=str(v))
            elif isinstance(v, QuerySet) or isinstance(v, Manager):
                v = [dict(id=v.id, value=str(v)) for v in v]
            elif isinstance(v, bool):
                v = str(v).lower()
            elif isinstance(v, datetime.datetime):
                v = v.strftime("%Y-%m-%d %H:%M")
            elif isinstance(v, datetime.date):
                v = v.strftime("%Y-%m-%d")
            self.controls["set"][k] = v

    def get(self, name, value, default=None):
        if value is None:
            return default
        else:
            try:
                value = self.form.fields[name].to_python(value)
            except KeyError:
                pass
            except ValidationError:
                pass
            return default if value is None else value

    def clear(self):
        self.controls["show"].clear()
        self.controls["hide"].clear()
        self.controls["set"].clear()

    def on_change(self, field_name):
        self.clear()
        getattr(self.form._endpoint, f"on_{field_name}_change")(self, self.values())
        return self.controls


    def get_field_queryset(self, fname, qs):
        method_attr = None
        method_name = f"get_{fname}_queryset"
        if hasattr(self.form._endpoint, method_name):
            method_attr = getattr(self.form._endpoint, method_name)
        elif hasattr(self.form, "instance") and hasattr(
            self.form.instance, method_name
        ):
            method_attr = getattr(self.form.instance, method_name)
        return method_attr(qs, self.values()) if method_attr else qs

    def values(self):
        data = dict(**self.controls["set"])
        for name in self.form.fields:
            value = self.get(name, self.form.request.GET.get(name))
            if value:
                data[name] = value
        return data


class FormMixin:

    def fieldset(self, title, fields):
        self.fieldsets[title] = fields
        return self

    def parse_json(self):
        content_type = self.request.META.get("CONTENT_TYPE")
        method = self.request.method.lower()
        if content_type and content_type.lower() in "application/json":
            if method == "get" or method == "post":
                d1 = json.loads(self.request.body.decode()) if self.request.body else {}
                d2 = self.request.GET if method == "get" else self.request.POST
                d2._mutable = True
                d2.clear()
                for k, v in d1.items():
                    if isinstance(v, list):
                        for i, item in enumerate(v):
                            prefix = f"{k}__{i}"
                            d2[f"{prefix}__id"] = item.get("id")
                            for k1, v1 in item.items():
                                d2[f"{prefix}__{k1}"] = v1
                    elif isinstance(v, dict):
                        prefix = k
                        d2[f"{prefix}__id"] = v.get("id")
                        for k1, v1 in v.items():
                            d2[f"{prefix}__{k1}"] = v1
                    else:
                        d2[k] = v
                d2._mutable = False

    @classmethod
    def get_metadata(cls, name, default=None):
        metaclass = getattr(cls, "Meta", None)
        return getattr(metaclass, name, default) if metaclass else default

    def serialize(self):
        return self.to_dict()

    def display(self, serializable):
        self._display = serializable
        return self

    def to_dict(self, prefix=None):
        field_name = self.request.GET.get("on_change")
        if field_name:
            raise JsonResponseException(self.controller.on_change(field_name))

        data = dict(
            type="form",
            key=self._key,
            method=self._method,
            title=self.get_metadata("title", self._title),
            icon=self.get_metadata("icon"),
            style=self.get_metadata("style"),
            url=absolute_url(self.request),
            info=self._info,
        )
        data.update(
            controls=self.controller.controls, width=self.get_metadata("width", "100%")
        )
        if self._display:
            if isinstance(self._display, Serializer):
                # self._display.request = self.request
                data.update(
                    display=self._display.serialize(forward_exception=True)["data"]
                )
        choices_field_name = self.request.GET.get("choices")
        if self.fieldsets:
            fieldsetlist = []
            for title, names in self.fieldsets.items():
                fields = []
                if prefix:
                    value = self.instance.pk if isinstance(self, ModelForm) else ""
                    fields.append(
                        [
                            dict(
                                type="hidden",
                                name=f"{prefix}__id",
                                value=value,
                                label="ID",
                            )
                        ]
                    )
                for name in names:
                    if isinstance(name, str):
                        field = self.serialize_field(
                            name, self.fields[name], prefix, choices_field_name
                        )
                        fields.append([field])
                    else:
                        fields.append(
                            [
                                self.serialize_field(
                                    name, self.fields[name], prefix, choices_field_name
                                )
                                for name in name
                            ]
                        )
                if (
                    len(fields) == 1
                    and isinstance(fields[0], list)
                    and fields[0][0]["type"] == "inline"
                ):
                    fieldsetlist.append(fields[0][0])
                else:
                    fieldsetlist.append(
                        dict(type="fieldset", title=title, fields=fields)
                    )
            data.update(fieldsets=fieldsetlist)
        else:
            fields = []
            if prefix:
                value = self.instance.pk if isinstance(self, ModelForm) else ""
                fields.append(
                    dict(type="hidden", name=f"{prefix}__id", value=value, label="ID")
                )
            for name, field in self.fields.items():
                fields.append(
                    self.serialize_field(name, field, prefix, choices_field_name)
                )
            data.update(fields=fields)
        return data

    def serialize_field(self, name, field, prefix, choices_field_name):
        if isinstance(field, InlineFormField) or isinstance(field, InlineModelField):
            value = []
            instances = []
            is_one_to_one = field.max == field.min == 1
            if isinstance(self, ModelForm):
                if isinstance(field, OneToOneField):
                    instances.append(
                        getattr(self.instance, name) if self.instance.id else None
                    )
                elif isinstance(field, OneToManyField):
                    instances.extend(
                        getattr(self.instance, name).all() if self.instance.id else ()
                    )
            for i, instance in enumerate(instances):
                prefix = name if is_one_to_one else f"{name}__{i}"
                kwargs = (
                    dict(instance=instance, endpoint=self._endpoint)
                    if isinstance(field, InlineModelField)
                    else dict(endpoint=self._endpoint)
                )
                value.append(field.form(**kwargs).to_dict(prefix=prefix))
            if not is_one_to_one:
                value.append(
                    field.form(endpoint=self._endpoint).to_dict(prefix=f"{name}__n")
                )
            required = getattr(field, "required2", field.required)
            label = field.label.title() if field.label else field.label
            data = dict(
                type="inline",
                min=field.min,
                max=field.max,
                name=name,
                count=len(instances),
                label=label,
                required=required,
                value=value,
            )
        else:
            extra = {}
            ftype = FIELD_TYPES.get(type(field).__name__, "text")
            if name in self._values:
                ftype = "hidden"
                value = self._values[name]
                value = value.pk if isinstance(value, Model) else value
            else:
                value = field.initial or self.initial.get(name)
                if callable(value):
                    value = value()

                if isinstance(field, ModelMultipleChoiceField) or isinstance(
                    field, DjangoModelMultipleChoiceField
                ):
                    value = (
                        [dict(id=obj.id, label=str(obj).strip()) for obj in value]
                        if value
                        else []
                    )
                elif isinstance(field, MultipleChoiceField) or isinstance(
                    field, DjangoMultipleChoiceField
                ):
                    value = value if value else []
                elif value and (
                    isinstance(field, ModelChoiceField)
                    or isinstance(field, DjangoModelChoiceField)
                ):
                    obj = field.queryset.get(pk=value)
                    value = dict(id=obj.id, label=str(obj).strip())
                elif isinstance(field, DjangoImageField):
                    value = build_url(self.request, value.url) if value else None
                    extra.update(
                        extensions=field.extensions,
                        width=field.width,
                        height=field.height,
                    )
                elif isinstance(field, DjangoFileField):
                    value = build_url(self.request, value.url) if value else None
                    extra.update(extensions=field.extensions, max_size=field.max_size)

            if isinstance(field.widget, HiddenInput):
                ftype = "hidden"
                value = value["id"] if isinstance(value, dict) else value

            fname = name if prefix is None else f"{prefix}__{name}"
            data = dict(
                type=ftype,
                name=fname,
                label=field.label,
                required=field.required,
                value=value,
                help_text=field.help_text,
                mask=None,
            )
            data.update(**extra)
            for word in MASKS:
                if word in name:
                    data.update(mask=MASKS[word])
            for word in ("password", "senha"):
                if word in name:
                    data.update(type="password")
            if isinstance(field, CharField) or isinstance(field, IntegerField):
                mask = getattr(field, "mask", None)
                if mask:
                    data.update(mask=mask)
            if isinstance(field, CharField) and isinstance(field.widget, Textarea):
                data.update(type="textarea")
            if ftype == "decimal":
                data.update(mask="decimal")
            elif ftype == "scheduler":
                data.update(scheduler=field.scheduler)
            elif ftype == "choice":
                if name in self.request.GET and not choices_field_name:
                    data.update(type="hidden", value=self.request.GET[name])
                else:
                    pick = getattr(field, "pick", False)
                    if choices_field_name == fname or (isinstance(field.choices, ModelChoiceIterator) and not pick):
                        if choices_field_name == fname:
                            qs = field.choices.queryset
                            qs = self.controller.get_field_queryset(fname, qs)
                            if "term" in self.request.GET:
                                qs = qs.apply_search(self.request.GET["term"])
                            raise JsonResponseException(
                                [dict(id=obj.id, value=str(obj).strip()) for obj in qs[0:(50 if pick else 25)]]
                            )
                        else:
                            data["choices"] = absolute_url(
                                self.request, f"choices={fname}"
                            )
                    else:
                        if isinstance(field.choices, ModelChoiceIterator):
                            data["choices"] = [
                                dict(id=obj.id, value=str(obj).strip())
                                for obj in self.controller.get_field_queryset(fname, field.choices.queryset)
                            ]
                        else:
                            data["choices"] = [
                                dict(id=str(k), value=v) for k, v in field.choices
                            ]
                    if pick:
                        data.update(pick=absolute_url(
                            self.request, f"choices={fname}"
                        ))

        attr_name = f"on_{name}_change"
        if hasattr(self._endpoint, attr_name):
            data["onchange"] = absolute_url(self.request, f"on_change={name}")
        if name in self._actions:
            cls = ENDPOINTS[self._actions[name]]
            if cls.instantiate(self.request, self).check_permission():
                data.update(
                    action=cls.get_api_metadata(
                        self.request, absolute_url(self.request)
                    )
                )
        return data

    def submit(self):
        data = {}
        errors = {}
        inline_fields = {
            name: field
            for name, field in self.fields.items()
            if isinstance(field, InlineFormField) or isinstance(field, InlineModelField)
        }
        self.is_valid()
        errors.update(self.errors)
        for inline_field_name, inline_field in inline_fields.items():
            data[inline_field_name] = []
            for i in range(0, inline_field.max):
                is_one_to_one = inline_field.max == inline_field.min == 1
                prefix = (
                    inline_field_name if is_one_to_one else f"{inline_field_name}__{i}"
                )
                inline_form_field_name = f"{prefix}__id"
                if inline_form_field_name in self.data:
                    inline_form_data = {}
                    pk = self.data.get(inline_form_field_name)
                    if pk:
                        pk = int(pk)
                        for name in inline_field.form.base_fields:
                            inline_form_field_name = f"{prefix}__{name}"
                            inline_form_data[name] = self.data.get(
                                inline_form_field_name
                            )
                            if (
                                self.request.FILES
                                and inline_form_field_name in self.request.FILES
                            ):
                                self.request.FILES._mutable = True
                                self.request.FILES[name] = self.request.FILES[
                                    inline_form_field_name
                                ]
                                self.request.FILES._mutable = False
                        instance = (
                            inline_field.form._meta.model.objects.get(pk=abs(pk))
                            if pk
                            else None
                        )
                        if pk < 0:
                            setattr(instance, "deleting", True)
                        inline_form = inline_field.form(
                            data=inline_form_data,
                            instance=instance,
                            request=self.request,
                        )
                        if inline_form.is_valid():
                            if isinstance(inline_form, DjangoModelForm):
                                data[inline_field_name].append(inline_form.instance)
                            else:
                                data[inline_field_name].append(inline_form.cleaned_data)
                        else:
                            errors.update(
                                {
                                    f"{prefix}__{name}": error
                                    for name, error in inline_form.errors.items()
                                }
                            )
        if errors:
            raise JsonResponseException(
                dict(type="error", text="Por favor, corrija os erros.", errors=errors)
            )
        else:
            data.update(
                {
                    field_name: self.cleaned_data.get(field_name)
                    for field_name in self.fields
                    if field_name not in inline_fields
                }
            )
            with transaction.atomic():
                self.cleaned_data = data
                if isinstance(self, DjangoModelForm):
                    for inline_field_name in inline_fields:
                        for obj in self.cleaned_data[inline_field_name]:
                            obj.save()
                            # set one-to-one
                            if hasattr(self.instance, f"{inline_field_name}_id"):
                                if hasattr(obj, "deleting"):
                                    setattr(self.instance, inline_field_name, None)
                                else:
                                    setattr(self.instance, inline_field_name, obj)
                    self.save()
                    for inline_field_name in inline_fields:
                        for obj in self.cleaned_data[inline_field_name]:
                            if hasattr(obj, "deleting"):
                                obj.delete()
                return self.cleaned_data

    def settitle(self, title):
        self._title = title
        return self

    def setvalue(self, **kwargs):
        self._values.update(**kwargs)
        return self

    def actions(self, **kwargs):
        self._actions.update(kwargs)
        return self


class Form(DjangoForm, FormMixin):

    def __init__(self, *args, endpoint=None, **kwargs):
        self._display = []
        self._endpoint = endpoint
        self._info = None
        self._message = None
        self._redirect = None
        self._dispose = False
        self._values = {}
        self._title = type(self).__name__
        self._actions = {}
        self._method = "POST"
        self._key = self._title.lower()

        self.fieldsets = {}
        self.fields = {}
        self.request = endpoint.request
        self.controller = FormController(self)
        self.instance = kwargs.pop("instance", None)

        self.parse_json()
        if "data" not in kwargs:
            if self.request.method.upper() == "GET":
                data = self.request.GET or None
            elif self.request.method.upper() == "POST":
                data = self.request.POST or {}
        else:
            data = kwargs["data"]
        kwargs.update(data=data)
        if self.request:
            kwargs.update(files=self.request.FILES or None)
        super().__init__(*args, **kwargs)


class ModelForm(DjangoModelForm, FormMixin):

    def __init__(self, instance=None, endpoint=None, **kwargs):
        self._display = []
        self._endpoint = endpoint
        self._info = None
        self._message = None
        self._redirect = None
        self._dispose = False
        self._values = {}
        self._title = type(self).__name__
        self._actions = {}
        self._method = "POST"
        self._key = self._title.lower()

        self.fieldsets = {}
        self.request = endpoint.request
        self.controller = FormController(self)

        self.parse_json()
        if "data" not in kwargs:
            if self.request.method.upper() == "GET":
                data = self.request.GET or None
            elif self.request.method.upper() == "POST":
                data = self.request.POST or {}
            else:
                data = None
        else:
            data = kwargs["data"]
        kwargs.update(data=data)
        if self.request:
            kwargs.update(files=self.request.FILES or None)
        super().__init__(instance=instance, **kwargs)


class InlineFormField(Field):
    def __init__(self, *args, form=None, min=1, max=3, **kwargs):
        self.form = form
        self.max = max
        self.min = min
        super().__init__(*args, **kwargs)
        self.required = False


class InlineModelField(Field):
    def __init__(
        self, model, *args, fields="__all__", exclude=(), min=1, max=3, **kwargs
    ):
        if fields == "__all__":
            self.form = modelform_factory(
                model, form=ModelForm, fields=fields, exclude=exclude
            )
        else:
            field_names = []
            for field_name in fields:
                if isinstance(field_name, str):
                    field_names.append(field_names)
                else:
                    field_names.extend(field_name)
            self.form = modelform_factory(
                model, form=ModelForm, fields=field_names, exclude=exclude
            )
            self.form.fieldsets = {None: fields}
        self.max = max
        self.min = min
        super().__init__(*args, **kwargs)
        self.required = False


class OneToManyField(InlineModelField):
    def __init__(self, model, fields="__all__", exclude=(), min=1, max=3, **kwargs):
        super().__init__(model, fields=fields, exclude=exclude, min=min, max=max)


class OneToOneField(InlineModelField):
    def __init__(self, model, fields="__all__", exclude=(), **kwargs):
        super().__init__(model, fields=fields, exclude=exclude, min=1, max=1)


class DecimalField(DecimalField):
    def to_python(self, value):
        if isinstance(value, str) and "," in value:
            value = value.replace(".", "").replace(",", ".")
        return super().to_python(value)


class ColorField(CharField):
    pass


class FileField(FileField):
    def __init__(self, *args, extensions=("pdf",), max_size=5, **kwargs):
        self.extensions = extensions
        self.max_size = max_size
        super().__init__(*args, **kwargs)


class ImageField(ImageField):
    def __init__(
        self,
        *args,
        extensions=("png", "jpg", "jpeg"),
        width=None,
        height=None,
        **kwargs,
    ):
        self.extensions = extensions
        if width or height:
            self.width = width or height
            self.height = height or width
        else:
            self.width = self.height = 500
        super().__init__(*args, **kwargs)


class ChoiceField(ChoiceField):

    def __init__(self, *args, **kwargs):
        self.pick = kwargs.pop("pick", False)
        super().__init__(*args, **kwargs)


class MultipleChoiceField(MultipleChoiceField):

    def __init__(self, *args, **kwargs):
        self.pick = kwargs.pop("pick", False)
        super().__init__(*args, **kwargs)


class ModelChoiceField(ModelChoiceField):

    def __init__(self, *args, **kwargs):
        self.pick = kwargs.pop("pick", False)
        super().__init__(*args, **kwargs)


class ModelMultipleChoiceField(ModelMultipleChoiceField):

    def __init__(self, *args, **kwargs):
        self.pick = kwargs.pop("pick", False)
        super().__init__(*args, **kwargs)


class SchedulerField(CharField):
    def __init__(self, *args, scheduler=None, **kwargs):
        self.scheduler = scheduler or Scheduler()
        super().__init__(*args, **kwargs)

    def clean(self, value):
        values = dict(select=[], deselect=[])
        if value:
            data = json.loads(value)
            for key in values.keys():
                for date, hour in data[key]:
                    data_string = "{} {}".format(date, hour)
                    values[key].append(datetime.datetime.strptime(data_string, "%d/%m/%Y %H:%M"))
        return values


FIELD_TYPES = {
    "CharField": "text",
    "EmailField": "email",
    "DecimalField": "decimal",
    "BooleanField": "boolean",
    "DateTimeField": "datetime",
    "DateField": "date",
    "IntegerField": "number",
    "ChoiceField": "choice",
    "TypedChoiceField": "choice",
    "ModelChoiceField": "choice",
    "MultipleChoiceField": "choice",
    "ModelMultipleChoiceField": "choice",
    "ColorField": "color",
    "FileField": "file",
    "ImageField": "file",
    "SchedulerField": "scheduler",
}



# Generated by Django 4.2.6 on 2024-04-28 06:44

import django.contrib.auth.models
from django.db import migrations
import slth


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('slth', '0005_alter_profile_photo'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
            ],
            options={
                'verbose_name': 'Usuário',
                'verbose_name_plural': 'Usuários',
                'proxy': True,
                'icon': 'users',
                'indexes': [],
                'constraints': [],
            },
            bases=('auth.user', slth.ModelMixin),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
    ]



import os
import binascii
from .db import models, meta
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.models import User
from django.apps import apps
from datetime import datetime
from django.utils.html import strip_tags
from django.core.mail import EmailMultiAlternatives
from slth import APPLICATON

class RoleQuerySet(models.QuerySet):

    def contains(self, *names):
        _names = getattr(self, '_names', None)
        if _names is None:
            _names = set(self.filter(name__in=names, active=True).values_list('name', flat=True))
            setattr(self, '_names', _names)
        for name in names:
            if name in _names:
                return True
        return False

    def active(self):
        return self.filter(active=True)

    def inactive(self):
        return self.filter(active=False)
    
    def debug(self):
        print('------- ROLES -----------')
        print('USERNAME\tNAME\tSCOPE')
        for role in self:
            print('{}\t{}\t{}'.format(role.username, role.name, role.get_scope_value()))
        print('-------------------------')

class Role(models.Model):
    username = models.CharField(max_length=50, db_index=True)
    name = models.CharField(max_length=50, db_index=True)
    scope = models.CharField(max_length=50, db_index=True, null=True)
    model = models.CharField(max_length=50, db_index=True, null=True)
    value = models.IntegerField('Value', db_index=True, null=True)
    active = models.BooleanField('Active', default=True, null=True)

    objects = RoleQuerySet()

    class Meta:
        verbose_name = 'Papel'
        verbose_name_plural = 'Papéis'

    def __str__(self):
        return self.get_description()

    def get_verbose_name(self):
        return APPLICATON['groups'].get(self.name, self.name)

    def get_scope_value(self):
        return apps.get_model(self.model).objects.get(pk=self.value) if self.model else None

    @meta('Descrição')
    def get_description(self):
        scope_value = self.get_scope_value()
        return '{} - {}'.format(self.get_verbose_name(), scope_value) if scope_value else self.get_verbose_name()


class EmailManager(models.Manager):
    def all(self):
        return self.order_by('-id')

    def send(self, to, subject, content, from_email=None):
        to = [to] if isinstance(to, str) else list(to)
        return self.create(from_email=from_email or 'no-replay@mail.com', to=', '.join(to), subject=subject, content=content)


class PushSubscription(models.Model):
    user = models.ForeignKey('auth.user', verbose_name='Usuário', on_delete=models.CASCADE)
    device = models.CharField(verbose_name='Dispositivo')
    data = models.JSONField(verbose_name='Dados da Inscrição')

    class Meta:
        verbose_name = 'Inscrição de Notificação'
        verbose_name_plural = 'Inscrições de Notificação'

    def notify(self, text):
        import os
        from pywebpush import webpush
        specification = API.instance()
        response = webpush(
            subscription_info=self.data, data='{}>>>{}'.format(specification.title, text),
            vapid_private_key=os.environ.get('VAPID_PRIVATE_KEY'),
            vapid_claims={"sub": "mailto:admin@admin.com"}
        )
        return response.status_code == 201


class Error(models.Model):
    user = models.OneToOneField('auth.user', verbose_name='Usuário', on_delete=models.CASCADE, null=True)
    date = models.DateTimeField('Data/Hora')
    traceback = models.TextField('Rastreamento')

    class Meta:
        verbose_name = 'Erro'
        verbose_name_plural = 'Erros'


class Email(models.Model):
    from_email = models.EmailField('Remetente')
    to = models.TextField('Destinatário', help_text='Separar endereços de e-mail por ",".')
    subject = models.CharField('Assunto')
    content = models.TextField('Conteúdo', formatted=True)
    sent_at = models.DateTimeField('Data/Hora', null=True)

    objects = EmailManager()

    class Meta:
        verbose_name = 'E-mail'
        verbose_name_plural = 'E-mails'

    def __str__(self):
        return self.subject

    def save(self, *args, **kwargs):
        to = [email.strip() for email in self.to.split(',')]
        msg = EmailMultiAlternatives(self.subject, strip_tags(self.content), self.from_email, to)
        msg.attach_alternative(self.content, "text/html")
        if self.sent_at is None:
            msg.send(fail_silently=True)
            self.sent_at = datetime.now()
        super().save(*args, **kwargs)


class Profile(models.Model):
    
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    photo = models.ImageField(verbose_name=_("Foto"), width=500, blank=True, null=True, extensions=['png', 'jpg', 'jpeg'])

    class Meta:
        verbose_name = _("Profile")
        verbose_name_plural = _("Profile")


class Token(models.Model):
    key = models.CharField(_("Key"), max_length=40, primary_key=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    created = models.DateTimeField(_("Created"), auto_now_add=True)

    class Meta:
        verbose_name = _("Token")
        verbose_name_plural = _("Tokens")

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = binascii.hexlify(os.urandom(20)).decode()
        return super().save(*args, **kwargs)

    def __str__(self):
        return self.key
    
class User(User):
    class Meta:
        icon = 'users'
        proxy = True
        verbose_name = 'Usuário'
        verbose_name_plural = 'Usuários'

    def formfactory(self):
        return (
            super().formfactory()
            .fieldset('Dados Gerais', (('first_name', 'last_name'), 'email'))
            .fieldset('Dados de Acesso', ('username', ('is_superuser', 'is_active'),))
        )
    
    def serializer(self):
        return (
            super().serializer()
            .fieldset('Dados Gerais', (('first_name', 'last_name'), 'email'))
            .fieldset('Dados de Acesso', ('username', ('is_superuser', 'is_active'),))
            .queryset('Notificação', 'get_push_subscriptions')
            .queryset('Papéis', 'get_roles')
        )
    
    @meta('Inscrições de Notificação')
    def get_push_subscriptions(self):
        return self.pushsubscription_set.fields('device')
    
    @meta('Papéis')
    def get_roles(self):
        return Role.objects.filter(username=self.username).fields('get_description').actions('delete')





import sys
import slth
import socket
import traceback
from django.conf import settings
from .models import Token
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from .exceptions import JsonResponseException
from .serializer import serialize
from .utils import build_url
from slth import APPLICATON
from django.shortcuts import render
from django.views.decorators.cache import never_cache, cache_control

@cache_control(max_age=0, no_cache=True, no_store=True, must_revalidate=True)
def index(request, path=None):
    vite = not socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect_ex(('127.0.0.1',5173))
    return render(request, 'index.html', dict(vite=vite, application=APPLICATON))

def service_worker(request):
    return render(request, 'service-worker.js', content_type='text/javascript')

@csrf_exempt
def dispatcher(request, **kwargs):
    if request.method == 'OPTIONS':
        return ApiResponse({})
    else:
        if(1 and 'application' not in request.path and 'test' not in sys.argv): import time; time.sleep(0)
        if 'HTTP_AUTHORIZATION' in request.META:
            token = Token.objects.filter(key=request.META['HTTP_AUTHORIZATION'].split()[1]).first()
            if token:
                request.user = token.user
        if request.path == '/':
            cls = slth.ENDPOINTS.get(slth.APPLICATON['index'])
            url = build_url(request, cls.get_api_url())
            return ApiResponse(dict(type='redirect', url=url))
        else:
            cls = slth.ENDPOINTS.get(request.path.split('/')[2])
            if cls:
                try:
                    endpoint = cls(*kwargs.values()).contextualize(request)
                    if True or endpoint.check_permission():
                        return endpoint.to_response()
                    return ApiResponse({}, status=403)
                except JsonResponseException as e:
                    return ApiResponse(e.data, safe=False)
                except Exception as e:
                    traceback.print_exc() 
                    return ApiResponse(data=dict(error=str(e)), safe=False, status=500)
            else:
                return ApiResponse({}, status=404)

class ApiResponse(JsonResponse):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self["Access-Control-Allow-Origin"] = "*"
        self["Access-Control-Allow-Headers"] = "*"
        self["Access-Control-Allow-Methods"] = "GET, POST, OPTIONS, PUT, DELETE, PATCH";
        self["Access-Control-Max-Age"] = "600"



# Generated by Django 4.2.7 on 2024-04-18 16:49

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('slth', '0002_email_role_pushsubscription_error'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Foto',
            new_name='Profile',
        ),
        migrations.AlterModelOptions(
            name='profile',
            options={'verbose_name': 'Profile', 'verbose_name_plural': 'Profile'},
        ),
    ]



from django.contrib.auth.models import User
from django.conf import settings
from django.db.models import Model, QuerySet
from django.db.models.fields import related_descriptors
from django.db.models.signals import m2m_changed, post_save, post_delete

ROLES = {}

def related_model(model, relation_name):
 attr = getattr(model, relation_name, None)
 if attr is None or relation_name.startswith('get_'):
  if relation_name.startswith('get_'):
      attr = getattr(model, relation_name)
  else:
      attr = getattr(model, f'get_{relation_name}')
  value = attr(model(id=0))
  if isinstance(value, QuerySet):
    return value.model
  elif isinstance(value,Model):
    return type(value)
 if isinstance(attr, related_descriptors.ForwardManyToOneDescriptor):
    return attr.field.related_model
 elif isinstance(attr, related_descriptors.ManyToManyDescriptor):
    return attr.field.related_model
 elif isinstance(attr, related_descriptors.ReverseManyToOneDescriptor):
    return attr.rel.related_model


def post_save_func(sender, **kwargs):
    from .models import Role
    pk = kwargs['instance'].pk
    model = '{}.{}'.format(sender._meta.app_label, sender._meta.model_name)
    existing = set(Role.objects.filter(model=model, value=pk).values_list('pk', flat=True))
    creating = set()
    for name, role in sender.__roles__.items():
        scopes = role.get('scopes')
        values = [role[k] for k in ('username', 'email', 'inactive', 'active') if k in role and role[k]]
        for item in sender.objects.filter(pk=pk).values(*values):
            username = item[role['username']]
            email = item[role['email']] if 'email' in role and role['email'] else ''
            active = item[role['active']] if 'active' in role and role['active'] else True
            inactive = item[role['inactive']] if 'inactive' in role and role['inactive'] else False
            if username is None or (inactive or not active):
                if scopes:
                    for scope in scopes.keys():
                        Role.objects.filter(username=username, name=name, scope=scope, model=model, value=pk).delete()
                else:
                    Role.objects.filter(username=username, name=name).delete()
            else:
                user = User.objects.filter(username=username).first()
                if user is None:
                    user = User.objects.create(username=username)
                    user.email = email
                    if hasattr(settings, 'DEFAULT_PASSWORD'):
                        user.set_password(settings.DEFAULT_PASSWORD(user))
                    else:
                        user.set_password('123')
                    user.save()
                else:
                    if email and not user.email:
                        user.email = email
                        user.save()
                if scopes:
                    for scope, lookup in scopes.items():
                        scope_model = sender if lookup in ('id', 'pk') else related_model(sender, lookup)
                        model = '{}.{}'.format(scope_model._meta.app_label, scope_model._meta.model_name)
                        for value in sender.objects.filter(pk=pk).values_list(lookup, flat=True):
                            creating.add(Role.objects.get_or_create(
                                username=username, name=name, model=model, scope=scope, value=value
                            )[0].id)
                else:
                    creating.add(Role.objects.get_or_create(
                        username=username, name=name, model=None, scope=None, value=None
                    )[0].id)
    Role.objects.filter(pk__in=existing-creating).delete()


def m2m_save_func(sender, **kwargs):
    if kwargs['action'].startswith('post_'):
        post_save_func(type(kwargs['instance']), instance=kwargs['instance'])


def post_delete_func(sender, **kwargs):
    from .models import Role
    pk = kwargs['instance'].pk
    model = '{}.{}'.format(sender._meta.app_label, sender._meta.model_name)
    Role.objects.filter(model=model, value=pk).delete()


def role(name, username, email=None, active=None, inactive=None, **scopes):
    def decorate(cls):
        if not hasattr(cls, '__roles__'):
            cls.__roles__ = {}
        metadata = dict(username=username, email=email, active=active, inactive=inactive, scopes=scopes)
        ROLES[name] = (cls, metadata)
        cls.__roles__[name] = metadata
        post_save.connect(post_save_func, cls)
        post_delete.connect(post_delete_func, cls)
        for field in cls._meta.many_to_many:
            m2m_changed.connect(m2m_save_func, sender=getattr(cls, field.name).through)

        return cls
    return decorate



from .models import Role
from django.contrib.auth.models import User


def active_role_names(user):
    names = getattr(user, '_active_roles_names', None)
    if names is None:
        names = Role.objects.filter(username=user.username, active=True).values_list('name', flat=True)
        setattr(user, '_active_roles_names', names)
    return names


def check_roles(lookups, user):
    if lookups is None:
        checked = True
    elif user.is_superuser:
        checked = True
    else:
        checked = False
        for name in lookups.keys():
            if name is None:
                checked = True
            elif name in active_role_names(user):
                checked = True
    return checked


def check_lookups(instance, lookups, user):
    queryset = type(instance).objects.filter(pk=instance.pk) if hasattr(instance, 'pk') else instance
    checked = user.is_superuser or apply_lookups(queryset, lookups, user).exists()
    return checked


def apply_lookups(queryset, lookups, user):
    if isinstance(user, str):
        user = User.objects.get(username=user)
    if user.is_superuser:
        return queryset
    qs = queryset.none()
    roles = Role.objects.filter(username=user.username, active=True)
    role_names = set(active_role_names(user))
    role_names.add(None)
    for role_name, lookup in lookups.items():
        if role_name in role_names:
            if lookup:
                for scope_lookup, scopename in lookup.items():
                    if scopename == 'username':
                        kwargs = {scope_lookup: user.username}
                    else:
                        pks = roles.filter(scope=scopename).values_list('value', flat=True)
                        kwargs = {'{}__in'.format(scope_lookup): pks}
                    qs = qs | queryset.filter(**kwargs)
            else:
                qs = queryset
                break
    return qs


from django.conf import settings

def build_url(request, path=None):
    return path or request.path if request else ''

def absolute_url(request, *querystrings):
    url = build_url(request)
    if url:
        querstring = request.META.get('QUERY_STRING')
        url = append_url(url, querstring)
        for querystring in querystrings:
            url = append_url(url, querystring)
    return url

def append_url(url, *querystrings):
    params = {}
    if '?' in url:
        url, querystring = url.split('?')
        if querystring:
            for param in querystring.split('&'):
                k, v = param.split('=')
                params[k] = v
    else:
        url, querystring = url, ''
    for querystring in querystrings:
        if querystring:
            for param in querystring.split('&'):
                k, v = param.split('=')
                params[k] = v
    if params:
        url = '{}?{}'.format(url, '&'.join([f'{k}={v}' for k, v in params.items()]))
    return url



import os
from setuptools import find_packages, setup

install_requires = open('requirements.txt').read().splitlines()

setup(
    name='pyslth',
    version='0.3.4',
    packages=find_packages(exclude=('xxx', 'xxx.*')),
    install_requires=install_requires,
    extras_require={
        'dev': []
    },
    include_package_data=True,
    license='BSD License',
    description='API generator based on yml file',
    long_description='',
    url='https://github.com/brenokcc',
    author='Breno Silva',
    author_email='brenokcc@yahoo.com.br',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



import json
from pywebpush import webpush
from slth import APPLICATON

def send_push_web_notification(user, title, message, url=None, icon=None):
    icon = icon or APPLICATON['icon']
    for subscription in user.pushsubscription_set.all():
        data = webpush(
            subscription_info=subscription.data,
            data=json.dumps(dict(title=title, message=message, url=url, icon=icon)),
            vapid_private_key="GoFJpuTAdhepzfxOHdrW7u2ONh7V8ZIjPkjgpWSS3ks",
            vapid_claims={"sub": "mailto:admin@admin.com"}
        )
        print(data)



# Generated by Django 4.2.7 on 2024-04-23 09:54

from django.db import migrations
import slth.db.models


class Migration(migrations.Migration):

    dependencies = [
        ('slth', '0004_alter_profile_photo'),
    ]

    operations = [
        migrations.AlterField(
            model_name='profile',
            name='photo',
            field=slth.db.models.ImageField(blank=True, null=True, upload_to='', verbose_name='Foto'),
        ),
    ]



# -*- coding: utf-8 -*-

import os

from django.conf import settings
from django.contrib.auth.models import User
from django.core.management import call_command
from django.core.management.base import BaseCommand
from django.utils import termcolors


def print_and_call(command, *args, **kwargs):
    kwargs.setdefault('interactive', True)
    print(termcolors.make_style(fg='cyan', opts=('bold',))('>>> {} {}{}'.format(
        command, ' '.join(args), ' '.join(['{}={}'.format(k, v) for k, v in list(kwargs.items())]))))
    call_command(command, *args, **kwargs)


class Command(BaseCommand):
    def handle(self, *args, **options):

        app_labels = []
        for app_label in settings.INSTALLED_APPS:
            app_labels.append(app_label.split('.')[-1])

        print_and_call('makemigrations', *app_labels)
        print_and_call('migrate')
        print_and_call('collectstatic', clear=True, verbosity=0, interactive=False)

        if not User.objects.exists():
            user = User.objects.create_superuser('admin')
            password = getattr(settings, 'DEFAULT_PASSWORD', lambda user: '123')(user)
            user.set_password(password)
            user.save()
            print('Superuser "admin" with password "{}" was created.'.format(password))
